#ifndef _HATPSTREAMS_H
#define _HATPSTREAMS_H

#include<map>

#include "hatpNode.hh"

/**
 * Representation of streams.
 * The streams structure represents the primitives actions linked by agent
 * A stream is a sequence of actions for one agent
 */
class hatpStreams {

	private:
		std::map<unsigned int, hatpNode*> nodes;
		std::vector<unsigned int> roots;
		std::map<unsigned int, int> ranks;
		//TODO std::map<unsigned int, unsigned int> ranks;
		int maxRank;
	
	protected:

		/**
		 * Used by the constructor to compute the rank of each node in the streams.
		 * To each node is associated a rank (0 for root) indicating its position in a stream
		 * @see hatpStreams()
		 * @param cnode The node for which to compute rank
		 */
		void computeNodeRank(hatpNode* cnode);
	
	public:
	
		/**
		 * Constructor of the Streams structure.
		 * @param r The list of root nodes ID
		 * @param n The list of all nodes ID
		 */
		hatpStreams(std::vector<unsigned int> r, std::vector<hatpNode*> n);

		/**
		 * Empty destructor.
		 */
		~hatpStreams();

		/**
		 * Get the root nodes ID of the streams.
		 * @see getRootNodes() for pointers
		 * @return The list of root nodes ID
		 */
		std::vector<unsigned int> getRootIDs() const;

		hatpNode* getNode(unsigned int id) const;

		/**
		 * Get the Root nodes of the streams.
		 * @see getRootsIDs() for IDs
		 * @return A list of pointers to root nodes of streams
		 */
		std::vector<hatpNode*> getRootNodes() const;
		
		/**
		 * Get the list of nodes contained in streams.
		 * @return A list of pointers to all nodes
		 */
		std::vector<hatpNode*> getNodes() const;

		/**
		 * Get the maximum node ranking.
		 * @see computeNodeRank()
		 * @return The max rank for all the streams
		 */
		unsigned int getMaxRank() const;

		/**
		 * Get The rank of a given node ID.
		 * @param n The node ID
		 * @return The rank of the specified ID node or -1
		 */
		int getRankOfNode(unsigned int n) const;
		//TODO unsigned int getRankOfNode(unsigned int n) const;

		/**
		 * Get all the nodes of a specified rank (one node per stream).
		 * @param r the rank
		 * @return A list of all nodes of rank r
		 */
		std::vector<hatpNode*> getNodesOfRank(int r) const;
		//TODO std::vector<hatpNode*> getNodesOfRank(unsigned int r) const;

		/**
		 * Get all the nodes for a specified agent.
		 * Corresponds to get all the nodes of a stream
		 * @param ag The name of the concerned agent
		 * @return A list of all the nodes for this agent
		 */
		std::vector<hatpNode*> getNodesForAgent(std::string ag) const;

		/**
		 * Get the list of agents involved in streams.
		 * The he number of agents returned may indicate the number of streams in this structure
		 * @return A list of agent names
		 */
		std::vector<std::string> getAgentsList() const;

		/**
		 * Get the causal links of current streams.
		 * @return The list of causal links under the form of pairs of nodes ID	
		 */
		std::vector<std::pair<unsigned int, unsigned int> > getStreamsCausalLinks() const;

		/**
		 * Get the streams structure under a text form.
		 * @return A string representing this streams structure
		 */
		std::string toString() const;
		
};

#endif
